﻿namespace Ozeki
{ 
    public record OzHfIOPerformanceEstimation(int ProgressPercentage, long AverageSpeed, TimeSpan ETA)
    {
        public string DownloadSpeedToString(int decimalPlaces = 0)
        {
            var value = AverageSpeed;
            if (decimalPlaces < 0) { throw new ArgumentOutOfRangeException("decimalPlaces"); }
            if (value == 0) { return string.Format("{0:n" + decimalPlaces + "} bytes", 0); }
            value = value * 8;
            var mag = (int)Math.Log(value, 1000);
            if (mag < 0)
            {
                mag = 0;
            }
            var adjustedSize = (decimal)value / (1L << (mag * 10));

            string[] SizeSuffixes = { "b/s", "Kb/s", "Mb/s", "Gb/s", "Tb/s", "Pb/s", "Eb/s", "Zb/s", "Yb/s" };
            return string.Format("{0:n" + decimalPlaces + "} {1}",
                adjustedSize,
                SizeSuffixes[mag]);
        }

        public string ETAToString()
        {
            return string.Format("{0:D2}:{1:D2}:{2:D2}", ETA.Hours, ETA.Minutes, ETA.Seconds);
        }
    }
}
