﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    internal class OzHfConnectionTester
    {
        private static readonly object _instanceLock = new object();

        private static bool _testingInProgress = false;
        private bool _testSuccessful = false;
        private int _maxRetry;
        private IOzHfProgressReport _report;
        public OzHfConnectionTester(int maxRetry, IOzHfProgressReport report)
        {
            _maxRetry = maxRetry;
            _report = report;
        }
        private void testConnection()
        {
            int offset = 5;
            int retry = 0;
            while (!OzHfNetworkOperation.IsHuggingFaceReachable() && retry < _maxRetry)
            {
                _report.HandleEvent(OzHfDownloadEvent.Error($"Huggingface.co can't be reached. Waiting for: {offset} seconds till retry"));
                Thread.Sleep(offset * 1000);
                offset = offset * 2 >= 300 ? 300 : offset * 2;
                retry++;
            }
            _testSuccessful = retry < _maxRetry;            
            _testingInProgress = false;
        }

        public void TestConnection()
        {
            Thread thread = new Thread(() => testConnection());
            if (!_testingInProgress)
            {
                lock (_instanceLock)
                {
                    if (!_testingInProgress)
                    {          
                        _testingInProgress = true;
                        _report.HandleEvent(OzHfDownloadEvent.Info("Testing connection"));
                        thread.Start();
                    }
                }

            }         
            while (_testingInProgress);

            if (!_testSuccessful)
            {
                throw new OzHfConnectionTestException();
            }
            if(thread.ThreadState != ThreadState.Unstarted)
            {
                thread.Join();
            }
        }
    }

    public class OzHfConnectionTestException : Exception
    {
        public OzHfConnectionTestException() : base("Can't connect to HuggingFace.co")
        {
        }
    }
}
