﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ozeki
{
    internal class OzHfThrottleShare
    {
        private readonly long _maxSpeedInBits;
        private int _numberOfThreads;

        private object _lock = new object();

        private Dictionary<string, Action<long>> _actions = [];
        public OzHfThrottleShare(long maxSpeedInBits)
        {
            _numberOfThreads = 0;
            _maxSpeedInBits = maxSpeedInBits;
        }

        public void UpdateThrottleShare()
        {
            if (_numberOfThreads > 0)
            {
                var throttleSpeed = (long)Math.Round((double)_maxSpeedInBits / _numberOfThreads);
                foreach (var action in _actions.Values)
                {
                    action(throttleSpeed);
                }
            }
        }

        public void Register(Action<long> updateCallback, string id)
        {
            lock (_lock)
            {
                _actions[id] = updateCallback;
                _numberOfThreads = _actions.Count;
                UpdateThrottleShare();
            }
        }

        public void Leave(string id)
        {
            lock (_lock)
            {
                if (_actions.Remove(id))
                {                
                    _numberOfThreads--;
                    UpdateThrottleShare();
                }
            }
        }
    }
}
